--@Author: Alex2009, Sven777b
--@Web: www.alex2009.de, www.LS-Landtechnik.com

floodlightPlaceable = {};
floodlightPlaceable_mt = Class(floodlightPlaceable, Placeable);
InitObjectClass(floodlightPlaceable, "floodlightPlaceable");

function floodlightPlaceable:new(isServer, isClient, customMt)
    local self = Placeable:new(isServer, isClient, floodlightPlaceable_mt);
    registerObjectClassName(self, "floodlightPlaceable");
    return self;
end;

function floodlightPlaceable:delete()
    removeTrigger(self.stateChange);
    unregisterObjectClassName(self);
    floodlightPlaceable:superClass().delete(self);
end;

function floodlightPlaceable:deleteFinal()
    floodlightPlaceable:superClass().deleteFinal(self);
end;

function floodlightPlaceable:load(xmlFilename, x,y,z, rx,ry,rz, moveMode, initRandom)
    if not floodlightPlaceable:superClass().load(self, xmlFilename, x,y,z, rx,ry,rz, moveMode, initRandom) then
        return false;
    end;
    local xml = loadXMLFile("TempXML", xmlFilename);
    self.light = Utils.indexToObject(self.nodeId, getXMLString(xml, "placeable.LightSettings#lightIndex"));
    self.Sublight = Utils.indexToObject(self.nodeId, getXMLString(xml, "placeable.LightSettings#subLightIndex"));
    self.stateChange = Utils.indexToObject(self.nodeId, getXMLString(xml, "placeable.LightSettings#stateIndex"));
    addTrigger(self.stateChange,"triggerCallback",self);
    setVisibility(self.light,false);
    setVisibility(self.Sublight,false);
    self.lightEnabled = false;
    self.changeAllowed = false;
    self.firstrun = false;
    delete(xml);
    return true;
end;

function floodlightPlaceable:loadFromAttributesAndNodes(xmlFile, key, reset)
    if not floodlightPlaceable:superClass().loadFromAttributesAndNodes(self, xmlFile, key, reset) then
        return false;
    end;
    local state = Utils.getNoNil(getXMLBool(xmlFile,key.."#lightState"),false);
    self.lightEnabled = state;
    return true;
end;

function floodlightPlaceable:getSaveAttributesAndNodes(nodeIdent)
    local attributes, nodes = floodlightPlaceable:superClass().getSaveAttributesAndNodes(self, nodeIdent);
    local attributes = attributes..'lightState="'..tostring(self.lightEnabled)..'"';
    local nodes = '';
    if nodes ~= '' then
        nodes = nodes..'';
    end
    return attributes, nodes;
end;

function floodlightPlaceable:update(dt)
    if not self.firstrun then
        self:SetFloodlightState(self.lightEnabled);
        self.firstrun=true;
    end;
    if self.changeAllowed then
        if self.lightEnabled then
            g_currentMission:addHelpButtonText(g_i18n:getText("DISABLE_PLACEABLE_LIGHT"), InputBinding.JUMP);
        else
            g_currentMission:addHelpButtonText(g_i18n:getText("ENABLE_PLACEABLE_LIGHT"), InputBinding.JUMP);
        end;
        if InputBinding.hasEvent(InputBinding.JUMP) then
            self:SetFloodlightState(not self.lightEnabled);
        end;
    end;
end;

function floodlightPlaceable:triggerCallback(triggerId,otherId,onEnter,onLeave,onStay)
    if (triggerId == self.stateChange) then
        if (onEnter and g_currentMission.controlPlayer and otherId == g_currentMission.player.rootNode) then
            self.changeAllowed = true;
        elseif (onLeave) then
            self.changeAllowed = false;
        end;
    end;
end;

function floodlightPlaceable:readStream(streamId, connection)
    if connection:getIsServer() then
        local state = streamReadBool(streamId)
        self:SetFloodlightState(state,true);
    end
end

function floodlightPlaceable:writeStream(streamId, connection)
    if not connection:getIsServer() then
        streamWriteBool(streamId, self.lightEnabled)
    end
end

function floodlightPlaceable:SetFloodlightState(state,mp)
    self.lightEnabled = state;
    setVisibility(self.light,state);
    setVisibility(self.Sublight,state);
    if state then
        setVisibility(self.light,true);
        setVisibility(self.Sublight,true);
    else
        setVisibility(self.light,false);
        setVisibility(self.Sublight,false);
    end;
    if mp == nil then
        SetFloodlightStateEvent.sendEvent(self, state);
    end;
end;
registerPlaceableType("floodlightPlaceable", floodlightPlaceable);

SetFloodlightStateEvent = {};
SetFloodlightStateEvent_mt = Class(SetFloodlightStateEvent, Event);
InitEventClass(SetFloodlightStateEvent, "SetFloodlightStateEvent");
function SetFloodlightStateEvent:emptyNew()
    local self = Event:new(SetFloodlightStateEvent_mt);
    self.className="SetFloodlightStateEvent";
    return self;
end;

function SetFloodlightStateEvent:new(vehicle, state)
    local self = SetFloodlightStateEvent:emptyNew()
    self.vehicle = vehicle;
	self.state = state;
    return self;
end;

function SetFloodlightStateEvent:readStream(streamId, connection)
    self.vehicle = networkGetObject(streamReadInt32(streamId));
	self.state = streamReadBool(streamId);
    self:run(connection);
end;

function SetFloodlightStateEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.vehicle));
	streamWriteBool(streamId, self.state);
end;

function SetFloodlightStateEvent:run(connection)
	self.vehicle:SetFloodlightState(self.state, true);
    if not connection:getIsServer() then
        g_server:broadcastEvent(SetFloodlightStateEvent:new(self.vehicle, self.state), nil, connection, self.vehicle);
    end;
end;

function SetFloodlightStateEvent.sendEvent(vehicle, state, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(SetFloodlightStateEvent:new(vehicle, state), nil, nil, vehicle);
		else
			g_client:getServerConnection():sendEvent(SetFloodlightStateEvent:new(vehicle, state));
		end;
	end;
end;